--[[
********************************************************
Script for USB RC-Remote Aurico RC-CHDK2
Requires CHDK 1.3 v3442 or later for USB high speed mode
Version: 1.10 Beta 1
Date: 2016/08/16
Licence: GPL
Author: Werner_O
THX to msl
********************************************************
@title RC-CHDK2 v1.10b1
@chdk_version 1.3

@param a Enable AutoMode USB-Port
 @default a 1
 @range a 0 1
@param s Disable Shoot
 @default s 0
 @range s 0 1
@param h Halfshoot/Video
 @default h 0
 @values h Half_1x HalfCon Vid_Rec
@param v Disable Halfshoot/Video
 @default v 0
 @range v 0 1
@param t Zoom Time (ms)
 @default t 10
 @range t 1 500
@param z Disable Zoom
 @default z 0
 @range z 0 1
@param b USB Time Limit 1 (10ms)
 @default b 13
 @range b 5 15
@param c USB Time Limit 2 (10ms)
 @default c 23
 @range c 10 24
@param d USB Time Limit 3 (10ms)
 @default d 33
 @range d 15 40
@param x Start in USB Test-Mode
 @default x 0
 @range x 0 1
@param p Result Report Time (s)
 @default p 3
 @range p 1 10
@param m Shoot Mode HalfShoot
 @default m 0
 @values m Press Click
]]

--Console layout:
set_console_layout(0, 0, 26, 8)

--Predefinition of the used variables:
SleepTime = p * 1000
ZoomTime = t
AutoMode = a
Limit_1 = b
Limit_2 = c
Limit_3 = d
HalfShoot = h
HalfShootState = 0
TestMode = x
ShootLock = s
VideoHalfshootLock = v
ZoomLock = z
zoom_step = get_zoom()
VideoState = 0
MaxZoom = get_zoom_steps() - 1     --amount of available zoom steps
separate_video_button = get_video_button()  --Is separate video button available at this CAM?
ShootModeTest = m         --Only for testing - will be removed in alpha version!

--Subfunction restore() used at script interrupt:
function restore()
    cls()
    print("Aborting Script:")
    if AutoMode == 1 then
        set_config_value(121,0)   --Disable USB remote
        print("USB remote disabled")
    end
    if high_speed then            --Disable USB high speed mode if automatically enabled
        if type(set_remote_timing) == "function" then set_remote_timing(0)  
        elseif type(enable_remote_hp_timer) == "function" then enable_remote_hp_timer(0)
        elseif type(enable_highspeed_usb) == "function" then enable_highspeed_usb(0)
        end
        print("High speed mode disabled")
    end
    print("Finished!")
    sleep(3000)
    console_redraw()
end

--Subfunction wait_message():
function wait_message()
    print("Waiting for USB Impulse...")
    print("(Press MENU to EXIT)")
    last_tick = get_tick_count()
end

--Subfunction hs_release_message():
function hs_release_message()
    if HalfShoot == 0 then
        print("Halfshoot (1x) released")
    else
        print("Halfshoot Cont. released")
    end
end

--Subfunction zoom_message():
function zoom_message()
    zoom_step = get_zoom()
    print(string.format("#Zoom-Step: %d of %d", zoom_step, MaxZoom))
end

--Subfunction command_shoot():
function command_shoot()
    cls()
    if ShootLock == 0 then            --true if shoot function is not disabled
        count = get_exp_count()
        if HalfShootState == 0 then   --true if halfshoot is not active
            shoot()
            repeat sleep(10) until get_exp_count() ~= count  --wait until camera has stored the picture
            print("Shooting successful")
        else                          --true if halfshoot is active
--Start Test Shoot Mode with active ShootHalf using alternate Shoot Modes
            if ShootModeTest == 0 then
                press("shoot_full_only")   --Try Press/Release
                repeat sleep(10) until get_exp_count() ~= count  --wait until camera has stored the picture
                release("shoot_full_only")
            else
                click("shoot_full_only")   --Try Click-Only
                repeat sleep(10) until get_exp_count() ~= count  --wait until camera has stored the picture
            end
--End Test
            print("Shooting successful")
            if HalfShoot == 0 then
                release("shoot_half")
                HalfShootState = 0
                hs_release_message()
            end
        end
    else
        print("ERROR: SHOOT IS DISABLED!")
    end
    wait_message()
end

--Subfunction command_video() (start/stop video):
function command_video()
    cls()
    if VideoHalfshootLock == 0 then       --true if video/halfshoot function is not disabled
        if separate_video_button == 1 then
            click("video")                --start/stop video
            sleep(2000)
            if VideoState == 0 then       --true if no video recording active
                print("Video Recording started")
                VideoState = 1
            else
                print("Video Recording stopped")
                VideoState = 0
            end
        else
            print("ERROR:")
            print("NO VIDEO BUTTON AVAILABLE!")
        end
    else
        print("Video Record is disabled!")
    end
    wait_message()
end

--Subfunction command_halfshoot() (enable/disable halfshoot):
function command_halfshoot()
    cls()
    if VideoHalfshootLock == 0 then       --true if video/halfshoot function is not disabled
        if HalfShootState == 0 then       --true if no halfshoot active
            press("shoot_half")
            repeat sleep(10) until get_shooting() == true    --wait until camera is ready
            HalfShootState = 1
            if HalfShoot == 0 then
                print("Halfshoot (1x) locked")
            else
                print("Halfshoot Cont. locked")
            end
        else
            release("shoot_half")
            HalfShootState = 0
            hs_release_message()
        end
    else
        print("HALFSHOOT IS DISABLED!")
    end
    wait_message()
end

--Subfunction command_zoomin():
function command_zoomin()
    cls()
    if ZoomLock == 0 then                 --true if zoom function is not disabled
        if HalfShootState == 1 then       --release halfshoot if currently aktiv
            release("shoot_half")
            HalfShootState = 0
            hs_release_message()
        end
        if get_zoom() < MaxZoom then      --is zoom position lower than max.?
            print(string.format("Zoom-In for %03d ms", ZoomTime))
            press("zoom_in")   
            sleep(ZoomTime)
            release("zoom_in")
            zoom_message()
        else
            zoom_message()
            print("MAX. ZOOM POSITION!")
        end
    else
        print("ERROR: ZOOM IS DISABLED!")
    end
    wait_message()
end

--Subfunction command_zoomout():
function command_zoomout()
    cls()
    if ZoomLock == 0 then                 --true if zoom function is not disabled
        if HalfShootState == 1 then       --release halfshoot if currently aktiv
            release("shoot_half")
            HalfShootState = 0
            hs_release_message()
        end
        if get_zoom() > 0 then            --is zoom position higher than min.?
            print(string.format("Zoom-Out for %03d ms", ZoomTime))
            press("zoom_out")   
            sleep(ZoomTime)
            release("zoom_out")
            zoom_message()
        else
            zoom_message()
            print("MIN. ZOOM POSITION!")
        end
    else
        print("ERROR: ZOOM IS DISABLED!")
    end
    wait_message()
end

--Start main script:

--Enable Shooting Mode when started in Playback Mode:
if get_mode() == false then       --True at found playback mode
    print("Activate Record Mode...")
    set_record(1)
    sleep(1000)
    while get_mode() == false do sleep(10) end
    print("Record Mode activated!")
    sleep(3000)
    cls()
end

--Abort script if Shoot AND Video Recording are disabled:
Ready = true
if ShootLock == 1 then
    if (VideoHalfshootLock == 1) or (HalfShoot <= 1) then
        Ready = false
    end
end

if Ready == true then
    print("Script started - State:")
    if AutoMode == 1 then
        set_config_value(121,1)        --enable USB remote
        print("USB remote: Enabled")
    end
    high_speed = false    --Enable USB high speed mode if possible, available since CHDK-DE 1.3 v3442
    --Command for CHDK 1.3 >= v3791 or CHDK 1.4/1.5 (boolean):
    if type(set_remote_timing) == "function" then high_speed = set_remote_timing(1000)
    --Command for CHDK 1.3 >= v3585 to < v3791 (boolean):
    elseif type(enable_remote_hp_timer) == "function" then high_speed = enable_remote_hp_timer(1000)
    --Command for CHDK 1.3 >= v3442 to < v3585 (number):
    elseif type(enable_highspeed_usb) == "function" then enable_highspeed_usb(1000) high_speed = true
    end
    if high_speed then
        Limit_1 = b * 10
        Limit_2 = c * 10
        Limit_3 = d * 10
        print("High speed mode: Enabled")
    else
        print("High speed mode: n/a")
    end
    
    if TestMode == 1 then              --messages for test mode
        print("USB Test Mode: Enabled")
        sleep(3000)
        console_redraw()
        sleep(3000)
        cls()
        print("USB Test Mode:")
        wait_message()
    else         --messages for standard mode:
        if ShootLock == 0 then
            print("Shoot: Enabled")
        else
            print("Shoot: Disabled")
        end

        if VideoHalfshootLock == 0 then
            if HalfShoot == 2 then
                if separate_video_button == 1 then
                    print("Video: Enabled")
                    print("Halfshoot: Disabled")
                else
                    print("ERROR: VIDEO ENABLED BUT")
                    print("NOT AVAILABLE AT THIS CAM!")
                    print("Halfshoot: Disabled")
                end
            elseif HalfShoot == 1 then
                print("Video: Disabled")
                print("Halfshoot Cont.: Enabled")
            else
                print("Video: Disabled")
                print("Halfshoot (1x): Enabled")
            end
        else
            print("Video: Disabled")
            print("Halfshoot: Disabled")
        end
        
        if ZoomLock == 0 then
            print("Zoom: Enabled")
        else
            print("Zoom: Disabled")
        end
        sleep(3000)
        console_redraw()
        sleep(3000)
        cls()
        wait_message()
    end

    repeat   --Start main loop
        current_tick = get_tick_count()
        if ((current_tick - last_tick) > 2900) then   --redraw console after 2.9 sec
            last_tick = current_tick
            console_redraw()
        end
        u = get_usb_power()
        if u > 0 then
            if TestMode == 1 then             --true in USB test mode
                cls()
                print("USB Impulse recognized:")
                if high_speed then
                    print(string.format("Impulse Length = %d ms", u))
                else
                    print(string.format("Impulse Length = %d0 ms", u))
                end
                print(string.format("(Script paused for %d sec)", p))
                sleep(SleepTime)              --pause to display result
                print("**************************")
                print("USB Test Mode:")
                wait_message()
            else                              --true in shooting mode
                if u < Limit_1 then           --true if USB impulse is shorter than Limit_1
                    command_shoot()
                elseif u < Limit_2 then       --true if USB impulse is shorter than Limit_2
                    if HalfShoot == 2 then    --true if Video recording is activated
                        command_video()
                    else
                        command_halfshoot()   --True if Halfshoot 1x/Cont. is activated
                    end
                elseif u < Limit_3 then       --true if USB impulse is shorter than Limit_3
                    command_zoomin()
                else                          --true if USB impulse is longer than Limit_3
                    command_zoomout()
                end
            end
        end
    until is_pressed("menu")
else
    cls()
    print("Neither Shoot nor Video")
    print("record are enabled currently.")
    print("Please reconfigure the")
    print("script settings in CHDK!")
end

restore()
